#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------


from os import access, R_OK, X_OK, listdir, stat
from os.path import exists, isdir, isfile, islink, isabs, join, sep

import tarfile

from xiv_diag.collector.collectable.exceptions import CollectionError, CollectionSkip
from xiv_xsf.console import print_in_color

from tarfile import os,sys,copy,shutil,stat,errno,time,struct,re,operator
from tarfile import BLOCKSIZE,NUL

MAX_READ_SIZE=512
MIN_READ_SIZE=1
ARCHIVE_MODE = 'w:gz'

class XIVDiagArchive(object):
    """
    The class for the diag output archive.
    Uses tarfile
    """
    def __init__(self, basedir, arcname, mode=ARCHIVE_MODE):
        """
        creates arcname.tar.gz in basedir
        """
        self.archive_dir = basedir
        self.archive_name = arcname
        self.archive_path = '%s.tar.gz' % join(basedir, arcname)
        self.tarfile_object = tarfile.open(name=self.archive_path, mode=mode)
   
    def _tarfile_add_wrap(self, name, recursive, exclude):
        """
        If TarFile.add raises IOError, the archive might get corrupted.
        This wrapper will fix the archive in such cases by writing zeros for the rest of the data length
        that is expected
        """
        arcname = join(self.archive_name, name) if not name.startswith(sep) else join(self.archive_name, name[1:])
        try:
            self.tarfile_object.add(name, arcname, recursive, exclude)
        except IOError, OSError:
            tarinfo = self.tarfile_object.gettarinfo(name, arcname)
            blocks, remainder = divmod(tarinfo.size, BLOCKSIZE)
            if remainder>0:
                blocks +=1
            missing_length = blocks*BLOCKSIZE - (self.tarfile_object.fileobj.tell() - self.tarfile_object.offset)
            self.tarfile_object.fileobj.write(NUL * (BLOCKSIZE*blocks))
            self.tarfile_object.offset += missing_length
            self.tarfile_object.members.append(tarinfo)
        
    def add(self, name, recursive=False, exclude=None):
        """
        add the given name under self.arcname
        name can be a file, directory, symlink, fifo, etc...
        """
        if not self.test_path(name, False):
            return
        if isdir(name):
            self._tarfile_add_wrap(name, False, exclude)
            for item in listdir(name):
                fullpath = join(name, item)
                if isdir(fullpath) and not recursive:
                    continue
                if isdir(fullpath) and islink(fullpath):
                    continue
                self.add(fullpath, recursive, exclude)
        else:
            self._tarfile_add_wrap(name, recursive, exclude)
            
    def close(self):
        self.tarfile_object.close()
        
    def _read_file(self, fileobj, length, event=None):
        """
            reads the data out of fileobj
            We don't want threads to get timed out during because of bad reads 
            and leave junk the archive, that is why we first test if we can read the file.
            it the threads hangs, it does not mess the archive.
        """
        BUFSIZE = 16 * 1024
        blocks, remainder = divmod(length, BUFSIZE)
        for b in xrange(blocks):
            buf = fileobj.read(BUFSIZE)
            if len(buf) < BUFSIZE:
                return 
            if not event is None:
                if event.is_set():
                    return
    
        if remainder != 0:
            buf = src.read(remainder)
            if len(buf) < remainder:
                return
        return

    def test_path(self, path, read=True, event=None):
        if not exists(path):
            return False
        if not access(path, R_OK):
            return False
        if isdir(path):
            if not access(path, X_OK):
                return False
            return True
        tarinfo = self.tarfile_object.gettarinfo(path)
        if tarinfo is None or not tarinfo.isreg():
            return True
        try:
            filesize = getattr(os.stat(path),'size') if hasattr(os.stat(path),'size') else 0
        except IOError, OSError:
            return False
        try:
            fileobj = open(path, 'rb')
        except IOError, OSError:
            return False
        if not isdir(path) and read and filesize:
            self._read_file(fileobj, filesize, event)
        fileobj.close()
        return True
