#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program  
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
# 
#-------------------------------------------------------------------------------------------------   

import os, time, zlib, platform, sys
from os import path
from xml.etree import ElementTree
from xml.etree.ElementTree import SubElement
from tempfile import mkdtemp, mktemp
from shutil import rmtree

from xiv_diag.collector.collectable.exceptions import CollectionError, CollectionSkip

from xiv_xsf.console import print_in_color

DEFAULT_DIR = '/tmp'

class XIVCollector(object):
    def __init__(self, default_dir = DEFAULT_DIR):
        self.attributes = dict()
        self.xml_root = ElementTree.Element("XcollectItem")
        self.add_attribute('run-date', '%s-%s-%s_%s-%s-%s' % (time.localtime()[0:6]))
        self.add_attribute('platform', platform.platform())
        
        self.default_dir = default_dir
        self.collectables = []
        self.tempdir = mkdtemp(prefix='xiv_diag_')

    def mktemp(self, suffix='', prefix=''):
        return mktemp(suffix, prefix, self.tempdir)

    def add_attribute(self, name, value):
        if self.attributes.has_key(name):
            raise AttributeError("Attribute %s already exists" % name)
        self.attributes[name] = str(value)
        attribute = SubElement(self.xml_root, name)
        attribute.text = str(value)

    def get_attribute(self, name):
        if not self.attributes.has_key(name):
                raise AttributeError("Attribute %s does not exist" % name)
        return self.attributes[name]

    def get_archive(self):
        self.archive = None
        self.arcname  = 'xiv_diag-results_%s' % self.get_attribute('run-date')
        while not self.archive:
            self.ask_for_directory()
            self.create_archive()

    def ask_for_directory(self):
        self.dirpath = raw_input('Please type in a path to place the xiv_diag file in [default: %s]: ' %  self.default_dir)
        if self.dirpath == '':
            self.dirpath = self.default_dir

    def create_archive(self):
        line = 'Creating archive %s ' % self.arcname
        print_in_color(line.ljust(70, ' '), newline=False)
        try:
            from xiv_diag.archive import XIVDiagArchive
            self.archive = XIVDiagArchive(self.dirpath, self.arcname)
        except Exception:
            print_in_color('ERROR'.rjust(9, ' '), 'red')
            print_in_color('ERROR: failed to create the archive in %s, please choose another one.' % self.dirpath, 'dark_red')
        print ''

    def collect_collectable(self, collectable):
        line = 'INFO: Gathering %s...     ' % (collectable.get_attribute('description'))
        print_in_color(line.ljust(70,' '), newline=False)
        try:
            collectable.collect()
            print_in_color('DONE'.rjust(9, ' '), 'dark_green')
        except CollectionSkip:
            print_in_color('SKIPPED'.rjust(9, ' '), 'yellow')
        except CollectionError, e:
            print_in_color('FAILED'.rjust(9, ' '), 'red')
            print_in_color('ERROR: Could not gather %s: (%s)' \
                           % (collectable.get_attribute('description'), str(e)), 'dark_red')
        self.add_collectable_elements_to_xml(collectable)

    def add_collectable_elements_to_xml(self, collectable):
        collectable_element = SubElement(self.xml_root, 'CollectableItem')
        for name, value in collectable.get_attributes_dictionary().iteritems():
            attribute = SubElement(collectable_element, name)
            attribute.text = str(value)

    def collect(self):
        for collectable in self.collectables:
            self.collect_collectable(collectable)
        self.write_xml_to_tempdir()
        self.add_tempdir_to_archive()

    def write_xml_to_tempdir(self):
        xml_filename = self.mktemp(prefix='xiv_diag_info', suffix='.xml')
        xml_file = open(xml_filename, 'w')
        xml_file.write(ElementTree.tostring(self.xml_root))
        xml_file.close()
    
    def add_tempdir_to_archive(self):
        self.archive.add(self.tempdir)

    def close(self):
        line = 'INFO: Closing xiv_diag archive file'
        print_in_color('\n' + line.ljust(70, ' '), newline=False)
        self.archive.close()
        print_in_color('DONE'.rjust(9, ' '), 'dark_green')
        line =  'Deleting temporary directory...'.ljust(70,' ')
        print_in_color(line.ljust(70, ' '), newline=False)
        rmtree(self.tempdir, ignore_errors=True)
        print_in_color('DONE'.rjust(9, ' '), 'dark_green')
        print 'INFO: Gathering is now complete.'
        print 'INFO: You can now send', self.archive.archive_path, 'to IBM-XIV for review.'
        print 'INFO: Exiting.'
        sys.exit(0)
    
    def add_collectable_item(self, collectable_item):
        self.collectables.append(collectable_item)
