#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program  
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
# 
#-------------------------------------------------------------------------------------------------   

import re, sys
from os import listdir, access, R_OK, X_OK, stat
from os.path import getmtime, join, exists, isfile, isdir, islink, sep, isdir
from time import localtime, strftime
from glob import glob

from interfaces import ICollectable
from exceptions import CollectionError, CollectionSkip, CollectorTimedOut
from collector_thread import CollectorThread

TIMEOUT = 120
READ_SIZE = 512

class SimpleDirectoryCollectable(ICollectable):
    """
    Collects all files within a directory, recursively by default
    """

    def __init__(self, description, archive, directory_to_collect, recursive_scan=True, regex_filter='.*', exclusions=[], timeout=TIMEOUT):
        ICollectable.__init__(self, description)
        self.archive = archive
        self.directory_to_collect = directory_to_collect
        self.add_attribute('type', 'directory')
        self.recursive_scan = recursive_scan
        self.regex_filter = regex_filter
        self.exclusions = exclusions
        self.collector_thread = CollectorThread(3*timeout)
        self.collect = self.collector_thread.thread(self)
        self.timeout = timeout
        self.event = None

    def _collect(self, event=None):
        read_thread = CollectorThread(self.timeout)
        try:
            read_thread.run(self, 'read_directory', self.directory_to_collect)
        except CollectorTimedOut:
            raise
        self.add_directory_to_archive()
        self.add_attribute("directory-name", self.directory_to_collect)
        self.add_attribute("exclusions", str(self.exclusions))
        self.add_attribute("recursive_scan", str(self.recursive_scan))
        self.add_attribute("regex-filter", str(self.regex_filter))
        self.add_attribute("collect-timestamp", strftime("%m/%d/%Y %I:%M:%S %p", localtime()))
    
    def read_directory(self, event, dirpath):
        if not exists(dirpath):
            raise CollectionSkip
        for item in listdir(dirpath):
            fullpath = join(dirpath, item)
            if self.exclude(fullpath):
                continue
            if event.is_set():
                return 
            self.archive.test_path(fullpath, True, event)
            if isdir(fullpath) and not islink(fullpath) and self.recursive_scan:
                self.read_directory(event, fullpath)

    def _is_included_in_exclusion_list(self, filepath):
        return filepath.split(sep)[-1] in self.exclusions

    def _is_excluded_by_regex_filter(self, filepath):
        result = re.match(self.regex_filter, filepath.split(sep)[-1])
        return True if result is None else False
    
    def exclude(self, path):
        if self._is_included_in_exclusion_list(path):
            return True
        if not isdir(path):
            if self._is_excluded_by_regex_filter(path):
                return True
        return False

    def add_path_to_archive(self, path, recursive=False):
        self.archive.add(path, recursive=recursive, exclude=self.exclude)

    def add_directory_to_archive(self):
        self.add_path_to_archive(self.directory_to_collect, self.recursive_scan)